
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <complex.h>
#include "serconv.h"
#include "nanovnaio.h"

// #define TTYDEV "/dev/ttyACM0"

int pause_cmd(char *ttydev)
{
	int fd;
	int bs;

	fd = open(ttydev, O_RDWR );
	if(fd < 0)
	{
		perror(ttydev);
		return(fd);
	}

	bs = pause_cmdfd(fd);

	close(fd);
	return(bs);
}


int pause_cmdfd(int fd)
{
	char reply[100];
	int bs;
	bs = conversatefd(fd, "pause\r\n", reply, 100); 
	// printf("Returned %d <%s>\n", bs, reply);
	return(bs);
}

int resume_cmd(char *ttydev)
{
	int fd;
	int bs;

	fd = open(ttydev, O_RDWR );
	if(fd < 0)
	{
		perror(ttydev);
		return(fd);
	}

	bs = resume_cmdfd(fd);

	close(fd);
	return(bs);
}


int resume_cmdfd(int fd)
{
	char reply[100];
	int bs;
	bs = conversatefd(fd, "resume\r\n", reply, 100); 
	// printf("Returned %d <%s>\n", bs, reply);
	return(bs);
}

int sweep_info(char *ttydev, uint32_t *fstartHz, uint32_t *fstopHz, int *fcount)
{
	int fd;
	int bs;

	fd = open(ttydev, O_RDWR );
	if(fd < 0)
	{
		perror(ttydev);
		return(fd);
	}

	bs = sweep_infofd(fd, fstartHz, fstopHz, fcount);

	close(fd);
	return(bs);
}


int sweep_infofd(int fd, uint32_t *fstartHz, uint32_t *fstopHz, int *fcount)
{
	char reply[128];
	int bs;
	char *stop;

	bs = conversatefd(fd, "sweep\r\n", reply, 128); 

	// printf("Returned %d <%s>\n", bs, reply);

	for(stop = reply; *stop; stop++)
	{
		if(isdigit(*stop)) break;
	}

	*fstartHz = strtol(stop, &stop, 10);
	*fstopHz = strtol(stop, &stop, 10);
	*fcount = strtol(stop, &stop, 10);

	return(bs);
}


int frequencies_info(char *ttydev, uint32_t *freqs, int fcount)
{
	int fd;
	int bs;

	fd = open(ttydev, O_RDWR );
	if(fd < 0)
	{
		perror(ttydev);
		return(fd);
	}

	bs = frequencies_infofd(fd, freqs, fcount);

	close(fd);
	return(bs);
}


int frequencies_infofd(int fd, uint32_t *freqs, int fcount)
{
	char cmd[128];
	char reply[200];
	int jf;
	char *stop;
	int cco;

	snprintf(cmd, 128, "frequencies\r\n");
	cco = strlen(cmd);
	write(fd, cmd, cco);

	/* Look at the reply. First line should be the echo. */
	stop = readln(fd, reply, 200);
	// printf("%s(%d): Returned %p %zd <%s>\n", __FILE__, __LINE__, stop, strlen(reply), reply);

	for(jf = 0; jf<fcount; jf++)
	{
		stop = readln(fd, reply, 200);
		// if(jf < 3)
		 // printf("%s(%d): Returned %p %zd <%s>\n", __FILE__, __LINE__, stop, strlen(reply), reply);

		freqs[jf] = strtol(stop, &stop, 10);
		// if(jf < 3)
		 // printf("%s(%d): From <%s> to %G\n", __FILE__, __LINE__, reply, (double) freqs[jf]);
	}
	return(0);
}


int data_info(char *ttydev, int channel, complex *Sxx, int fcount)
{
	int fd;
	int bs;

	fd = open(ttydev, O_RDWR );
	if(fd < 0)
	{
		perror(ttydev);
		return(fd);
	}

	bs = data_infofd(fd, channel, Sxx, fcount);

	close(fd);
	return(bs);
}



int data_infofd(int fd, int channel, complex *Sxx, int fcount)
{
	char cmd[128];
	char reply[200];
	int jf;
	char *stop;
	double re, im;
	int cco;

	snprintf(cmd, 128, "data %d\r\n", channel & 1);
	cco = strlen(cmd);
	write(fd, cmd, cco);

	/* Look at the reply. First line should be the echo. */
	stop = readln(fd, reply, 200);
	// printf("%s(%d): Returned %p %zd <%s>\n", __FILE__, __LINE__, stop, strlen(reply), reply);

	for(jf = 0; jf<fcount; jf++)
	{
		stop = readln(fd, reply, 200);
		// if(jf < 3)
		// printf("%s(%d): Returned %p %zd <%s>\n", __FILE__, __LINE__, stop, strlen(reply), reply);

		re = strtod(reply, &stop);
		im = strtod(stop, &stop);
		Sxx[jf] = re + I * im;

		// if(jf < 3)
		// printf("%s(%d): From <%s> %G and %G\n", __FILE__, __LINE__, reply, re, im);
	}
	return(0);
}

/* Go through the whole thing in sequence. fpcount should be the expected length of the data set
	as found via the sweep info command. */
int pullalldatas(char *ttydev, uint32_t *ftable, complex *S11, complex *S21, int fpcount)
{
	int fd;

	fd = open(ttydev, O_RDWR );
	if(fd < 0)
	{
		perror(ttydev);
		return(fd);
	}

	pause_cmdfd(fd);
	frequencies_infofd(fd, ftable, fpcount);
	data_infofd(fd, 0, S11, fpcount);
	data_infofd(fd, 1, S21, fpcount);
	resume_cmdfd(fd);

	close(fd);
	return(0);
}




