
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <complex.h>
#include "serconv.h"
#include "nanovnaio.h"

#define SHOWDB 1
#define SHOWMAG 2
#define SHOWRECT 4

int main(int argc, char **argv)
{
	uint32_t ftable[200];
	complex S11[120];
	complex S21[120];
	int jf, j;
	int showmode;
	int fd;

	uint32_t fstart, fstop;
	int fpcount;

	showmode = SHOWDB;

	for(j = 1; j<argc; j++)
	{
		if(!strcmp("-m", argv[j]))
		{
			showmode = SHOWMAG;
		}

		if(!strcmp("-d", argv[j]))
		{
			showmode = SHOWDB;
		}

		if(!strcmp("-r", argv[j]))
		{
			showmode = SHOWRECT;
		}
	}

	fd = open(TTYDEV, O_RDWR );
	if(fd < 0)
	{
		perror(TTYDEV);
		return(fd);
	}

	pause_cmdfd(fd);
	sweep_infofd(fd, &fstart, &fstop, &fpcount);
	frequencies_infofd(fd, ftable, fpcount);
	data_infofd(fd, 0, S11, fpcount);
	data_infofd(fd, 1, S21, fpcount);
	resume_cmdfd(fd);

	close(fd);

	printf("Sweep from %d to %d in %d steps\n", fstart, fstop, fpcount);

#if(1)
	if(showmode & SHOWMAG)
	{
		for(jf = 0; jf < fpcount; jf++)
		{
			printf("%d: F=%d S11=%G<%G S21=%G<%G\n", 
				jf, ftable[jf], 
				cabs(S11[jf]), 180.0/M_PI * carg(S11[jf]), 
				cabs(S21[jf]), 180.0/M_PI * carg(S21[jf]) ); 
		}
	}

	if(showmode & SHOWRECT)
	{
		for(jf = 0; jf < fpcount; jf++)
		{
			printf("%d: F=%d S11=%G %G S21=%G %G\n", 
				jf, ftable[jf], 
				creal(S11[jf]), cimag(S11[jf]), 
				creal(S21[jf]), cimag(S21[jf]) ); 
		}
	}

	if(showmode & SHOWDB)
	{
		for(jf = 0; jf < fpcount; jf++)
		{
			printf("%d: F=%d S11=%GdB<%G S21=%GdB<%G\n", 
				jf, ftable[jf], 
				20.0*log10(cabs(S11[jf])), 180.0/M_PI * carg(S11[jf]), 
				20.0*log10(cabs(S21[jf])), 180.0/M_PI * carg(S21[jf])); 
		}
	}
#endif

	return(0);
}

